from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required

from views import MyTicketListView, MyTicketDetailView, TicketCreateView

urlpatterns = patterns('',
    url(r'^my/$', login_required(MyTicketListView.as_view()), name='list'),
    url(r'^my/(?P<pk>\d+)/$', login_required(MyTicketDetailView.as_view()), name='detail'),
    url(r'^create/$', login_required(TicketCreateView.as_view()), name='create'),
)



from django.conf import settings
from django.utils.translation import ugettext_lazy as _


# List of available status options
STATUS_CHOICES = getattr(settings, 'TICKETS_STATUS_CHOICES', (
    (0, _("New")),
    (1, _("Feedback")),
    (3, _("In Progress")),
    (4, _("Resolved")),
    (5, _("Closed")),
    ))

# List of statuses that define a ticket as finally closed
CLOSED_STATUSES = getattr(settings, 'TICKETS_CLOSED_STATUSES', (4,5))




from django.test import TestCase

from tickets.tests.factories import UserFactory, TicketFactory, TicketCommentFactory


class TicketTestCase(TestCase):
    def test_model(self):
        obj = TicketFactory()
        self.assertTrue(obj.pk)

    def test_is_closed(self):
        obj = TicketFactory()
        self.assertFalse(obj.is_closed())
        obj.status = 4
        self.assertTrue(obj.is_closed())

class TicketCommentTestCase(TestCase):
    def test_model(self):
        obj = TicketCommentFactory()
        self.assertTrue(obj.pk)




from django import forms

from models import Ticket, TicketComment


class TicketCreateForm(forms.ModelForm):
    class Meta:
        model = Ticket
        fields = ('subject', 'description')

class TicketCommentCreateForm(forms.ModelForm):
    class Meta:
        model = TicketComment
        fields = ('comment',)



from django.views.generic import ListView, DetailView, CreateView
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from forms import TicketCreateForm, TicketCommentCreateForm
from models import Ticket


class MyTicketListView(ListView):
    model = Ticket
    template_name = 'tickets/ticket_mylist.html'

    def get_queryset(self):
        return Ticket.objects.filter(creator=self.request.user)

class MyTicketDetailView(DetailView):
    model = Ticket
    template_name = 'tickets/ticket_mydetail.html'
    form = TicketCommentCreateForm()

    def get_context_data(self, **kwargs):
        context = super(MyTicketDetailView, self).get_context_data(**kwargs)
        context['form'] = self.form
        return context

    def post(self, request, *args, **kwargs):
        self.form = TicketCommentCreateForm(request.POST)
        if self.form.is_valid():
            self.form_valid(self.form)
            return HttpResponseRedirect(reverse('tickets:detail', args=[self.kwargs['pk']]))
        return super(MyTicketDetailView, self).get(request, *args, **kwargs)

    def form_valid(self, form):
        comment = form.instance
        comment.author = self.request.user
        comment.ticket = Ticket.objects.get(id=self.kwargs['pk'])
        comment.save()
        messages.success(self.request, _("Ticket comment created."))

    def get_queryset(self):
        return Ticket.objects.filter(creator=self.request.user)

class TicketCreateView(CreateView):
    form_class = TicketCreateForm
    template_name = 'tickets/ticket_create.html'

    def form_valid(self, form):
        ticket = form.instance
        ticket.creator = self.request.user
        ticket.save()
        self.success_url = reverse('tickets:detail', args=[ticket.id])
        messages.success(self.request, _("Ticket created."))
        return super(TicketCreateView, self).form_valid(form)



from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _

from tickets import settings


class Ticket(models.Model):
    creator = models.ForeignKey(User, verbose_name=_("Creator"), related_name='tickets')
    date = models.DateTimeField(_("Date"), auto_now_add=True)
    last_update = models.DateTimeField(_("Date"), auto_now=True)
    subject = models.CharField(_("Subject"), max_length=255)
    description = models.TextField(_("Description"), help_text=_("A detailed description of your problem."))
    assignee = models.ForeignKey(User, verbose_name=_("Assignee"), related_name="assigned_tickets", blank=True, null=True)
    status = models.SmallIntegerField(_("Status"), choices=settings.STATUS_CHOICES, default=0)

    class meta:
        verbose_name = _("Ticket")
        verbose_name_plural = _("Tickets")
        ordering = ['date']

    def get_comments_count(self):
        return self.comments.count()

    def __unicode__(self):
        return "%s# %s" % (self.id, self.subject)

    def is_closed(self):
        return self.status in settings.CLOSED_STATUSES

class TicketComment(models.Model):
    ticket = models.ForeignKey(Ticket, verbose_name=_("Ticket"), related_name='comments')
    date = models.DateTimeField(auto_now_add=True, verbose_name=_("Date"), editable=False)
    author = models.ForeignKey(User, verbose_name=_("Author"))
    comment = models.TextField(_("Comment"))

    class Meta:
       verbose_name = _("Ticket comment")
       verbose_name_plural = _("Ticket comments")
       ordering = ['date']

    def __unicode__(self):
        return "Comment on " + unicode(self.ticket)




import os
from setuptools import setup, find_packages

import tickets


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django-tickets',
    version=tickets.__version__,
    description='Reusable django application providing a generic support ticket system',
    long_description=read('README.md'),
    license='MIT License',
    author='akuryou',
    author_email='contact@byteweaver.net',
    url='https://github.com/byteweaver/django-tickets',
    packages=find_packages(),
    install_requires=[
        'Django',
    ],
    tests_require=[
        'django-nose',
        'coverage',
        'django-coverage',
        'factory_boy',
    ],
)



from django.contrib.auth.models import User
import factory

from tickets.models import Ticket, TicketComment


class UserFactory(factory.Factory):
    FACTORY_FOR = User

    username = factory.Sequence(lambda n: "User %s" % n)

class TicketFactory(factory.Factory):
    FACTORY_FOR = Ticket
    
    creator = factory.LazyAttribute(lambda a: UserFactory())

class TicketCommentFactory(factory.Factory):
    FACTORY_FOR = TicketComment

    ticket = factory.LazyAttribute(lambda a: TicketFactory())
    author = factory.LazyAttribute(lambda a: UserFactory())




from django.contrib import admin

from models import Ticket, TicketComment


class TicketCommentInline(admin.StackedInline):
    model = TicketComment
    extra = 1


class TicketAdmin(admin.ModelAdmin):
    model = Ticket
    inlines = [TicketCommentInline,]


admin.site.register(Ticket, TicketAdmin)
